/*
 * CrudContainer
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.framework.criteria.rules;

import dk.apaq.framework.criteria.Rule.FiltrationItem;



/**
 *
 */
public class CompareRule<T> extends AbstractSpecificRule {

    private final CompareType compareType;
    private final T value;

    public enum CompareType {

        Equals, LessThan, LessOrEqual, GreaterThan, GreaterOrEqual
    }

    public CompareRule(String propertyId, T value, CompareType compareType) {
        super(propertyId);
        this.value = value;
        this.compareType = compareType;
    }

    public T getValue() {
        return value;
    }

    public CompareType getCompareType() {
        return compareType;
    }

    @Override
    public boolean passesRule(FiltrationItem item) {
        Object value = item.getFiltrationProperty(getPropertyId());
        switch (getCompareType()) {
            case Equals:
                return ( null == this.value ) ? ( null == value ) : this.value.equals(value);
            case GreaterThan:
                return compareValue(value) > 0;
            case LessThan:
                return compareValue(value) < 0;
            case GreaterOrEqual:
                return compareValue(value) >= 0;
            case LessOrEqual:
                return compareValue(value) <= 0;
        }
        // all cases should have been processed above
        return false;
    }

    @SuppressWarnings({"unchecked", "rawtypes"})
    protected int compareValue(Object value1) {
        if (null == value) {
            return null == value1 ? 0 : -1;
        } else if (null == value1) {
            return 1;
        } else if (value1 instanceof Comparable
                && value1.getClass().isAssignableFrom(getValue().getClass())) {
            return -( (Comparable) value1 ).compareTo(getValue());
        } else if (getValue() instanceof Number && value1 instanceof Number) {
            Double tmp = ((Number)value1).doubleValue();
            return tmp.compareTo(((Number)getValue()).doubleValue());
        }
        throw new IllegalArgumentException("Could not compare the arguments: "
                + value1 + "("+value1.getClass()+"), " + getValue() + "("+getValue().getClass()+")");
    }
}
